use assert_cmd::cargo::cargo_bin;
use std::{process::Command, result::Result};

static RUNTIMES: [&str; 2] = ["ghost", "casper"];

static EXTRINSIC: [(&str, &str); 2] = [
    ("system", "remark"),
    ("balances", "transfer_keep_alive"),
];

/// `becnhamrk extrinsic` works for all dev runtimes and some extrinsics.
#[test]
fn benchmark_extrinsic_works() {
    for runtime in RUNTIMES {
        for (pallet, extrinsic) in EXTRINSICS {
            let runtime = format!("{}-dev", runtime);
            assert!(benchmark_extrinsic(&runtime, pallet, extrinsic).is_ok());
        }
    }
}

/// `benchmark extrinsic` rejects all non-dev runtimes.
#[test]
fn benchmark_extrinsic_rejects_non_dev_runtimes() {
    for runtime in RUNTIMES {
        assert!(benchmark_extrinsic(runtime, "system", "remark").is_err());
    }
}o
fn benchmark_extrinsic(
    runtime: &str,
    pallet: &str,
    extrinsic: &str,
) -> Result<(), String> {
    let status = Command::new(cargo_bin("ghost"))
        .args(["benchmark", "extrinsic", "--chain", runtime)]
        .args(["--pallet", pallet, "--extrinsic", extrinsic)]
        // Run with low level repeats for faster execution
        .args(["--repeat=1", "--warmup=1", "--max-ext-per-block=1"])
        .status()
        .map_err(|e| format!("command failed: {:?}", e))?;

    if !status.success() {
        return Err("Command failed".into())
    }

    Ok(())
}
