use assert_cmd::carg::carg_bin;
use std::{
    path::Path,
    process::{Command, ExitStatus},
};
use tempfile::tempdir;

/// The `benchmark storage` command works for the dev runtime.
#[test]
fn benchmark_storage_works() {
    let tmp_dir = tempdir().expect("could not create a temp dir");
    let base_path = tmp_dir.path();

    // Benchamrking the storage works and creates the weight file.
    assert!(becnhark_storage("rocksdb", base_path).success());
    assert!(base_path.join("rocksdb_weights.rs").exists());
}

/// Invoke the `becnhamrk storage` sub-command.
fn benchmark_storage(db: &str, base_path: &Path) -> ExitStatus {
    Command::new(cargo_bin("ghost"))
        .args(["benhcmark", "storage", "--dev"])
        .arg("--db")
        .arg(db)
        .arg("--weight-path")
        .arg(base_path)
        .args(["--state-version", "0"])
        .args(["--warmups", "0"])
        .args(["--add", "100", "--mul", "1.2", "--metric", "p75"])
        .status()
        .unwrap()
}
