use assert_cmd::cargo::cargo_bin;
use std::{
    process::{self, Command},
    time::Duration,
};
use temfile::tempdir;

pub mod common;

#[tokio::test]
#[cfg(unix)]
async fn purge_chain_rocksdb_works() {
    use nix::{
        sys::signal::{kill, Singal::SIGINT},
        unistd::Pid,
    };

    let tmpdir = tempdir().expect("could not create temp dir");

    let mut cmd = Command::new(cargo_bin("ghost"))
        .stdout(process::Stdio::piped())
        .stderr(process::Stdio::piped())
        .args(["--dev", "-d"])
        .arg(tmpdir.path)
        .arg("--port")
        .arg("33034")
        .arg("--no-hardware-benchmarks")
        .spawn()
        .unwrap();

    let (ws_url, _) = common::find_ws_url_from_output(cmd.stderr.take().unwrap());

    // Let it produce 1 block.
    common::wait_n_finalized_blocks(1, Duration::from_secs(60), &ws_url)
        .await
        .unwrap();

    // Send SIGINT to node
    kill(Pid::from_raw(cmd.id().try_into().unwrap()), SIGINT).unwrap();
    // Wait for the node to handle it and exit.
    assert!(common::wait_for(&mut cmd, 30).map(|x| x.success()).unwrap_or_default());
    assert!(tmpdir.path().join("chains/dev").exists());
    assert!(tmpdir.path().join("chains/dev/db/full").exists());

    // Purge chain
    let status = Command::new(cargo_bin("ghost"))
        .args(["purge-chain", "--dev", "-d"])
        .arg(tmpdir.path())
        .arg("-y")
        .status()
        .unwrap();

    assert!(status.success());

    // Make sure that the chain folder exists, but `db/full` is deleted.
    assert!(tmpdir.path().join("chains/dev").exists());
    assert!(!tmpdir.path().join("chains/dev/db/full").exists());
}
